import 'package:flutter/widgets.dart';
import 'package:flutter_extensions/src/dart/MapExt.dart';
import 'package:flutter_extensions/src/widgets/widget_child.dart';
import 'package:provider/provider.dart';

/// Beta
class FocusDispatcher {
  Map<Object, FocusNode> _focusNodes;

  FocusDispatcher([Map<Object, FocusNode> focusNodes]) : _focusNodes = focusNodes ?? {};

  FocusNode operator [](Object key) => _focusNodes[key];

  void operator []=(Object key, FocusNode focusNode) => _focusNodes[key] = focusNode;

  FocusNode get(Object key) => _focusNodes[key];

  FocusNode getNext(Object key) => _focusNodes[_focusNodes.getKeyAfter(key)];

  FocusNode putIfAbsent(
    Object key, {
    String debugLabel,
    FocusOnKeyCallback onKey,
    bool skipTraversal = false,
    bool canRequestFocus = true,
  }) =>
      _focusNodes.putIfAbsent(
          key,
          () => FocusNode(
                debugLabel: debugLabel ?? '$key',
                onKey: onKey,
                skipTraversal: skipTraversal,
                canRequestFocus: canRequestFocus,
              ));

  void add(
    Object key, {
    String debugLabel,
    FocusOnKeyCallback onKey,
    bool skipTraversal = false,
    bool canRequestFocus = true,
  }) =>
      _focusNodes[key] = FocusNode(
        debugLabel: debugLabel ?? '$key',
        onKey: onKey,
        skipTraversal: skipTraversal,
        canRequestFocus: canRequestFocus,
      );

  void addAll(Map<Object, FocusNode> other) => _focusNodes.addAll(other);

  FocusNode remove(Object key) => _focusNodes.remove(key);

  void clear() {
    for (var focusNode in _focusNodes.values) {
      focusNode.dispose();
    }
  }

  void dispose() {
    clear();
    _focusNodes = null;
  }
}

/// Beta
mixin NestedFocusDispatcher on NestedChild {
  FocusDispatcher onCreateFocusDispatcher(BuildContext context) => FocusDispatcher();

  Widget buildWithChild(BuildContext context, Widget child) {
    return Provider<FocusDispatcher>(
      create: onCreateFocusDispatcher,
      dispose: (context, dispatcher) => dispatcher.dispose(),
      child: super.buildWithChild(context, child),
    );
  }

  FocusDispatcher focusDispatcherOf(BuildContext context) =>
      Provider.of<FocusDispatcher>(context, listen: false);
}
